//  ....................................................................
//
//  File: Printer.m   program: Objc_HelloWorld
//
//  This IMPLEMENTATION FILE, Printer.m, DEFINES the 'Printer' 
//  class object. 'define' just means Printer.m contains the 
//  code for the 'methods' (similar to functions) that the Printer
//  class uses to do things.
//  
//  Every class object in an Objective C program must have an
//  interface file (foo.h) to DECLARE the class and an implementation
//  file (foo.m) to DEFINE the class.
//
//  ....................................................................

//  Bring in the interface file.
#import "Printer.h"

//  Bring in necessary C and Objective-C library functions.
#import <string.h>
#import <objc/objc-api.h>

//  methods are defined between @implementation and @end.
@implementation Printer

//  The -init method: initializes a printer object.
- init
{
  //  'self' is the object and 'super' (for superclass) is the class 
  //  the object belongs to. In this method, self is printer and
  //  super is Printer.

  // This message tells printer to initialize itself using the -init
  // method of its superclass. 
  [super init];

  // To understand the above message, be aware that Object, Printer,
  // and printer each have their own -init methods.  The -init
  // method of any object normally overrides the -init method of the class
  // it belongs to. The message [super init] suppresses this override
  // and allows the superclass -init to initialize the object. The effect
  // is to give the object a set of default values from it's super.
  // The object's own -init can add extra initialization steps if 
  // desired, and in fact does so below.
  
  // This message tells newly-created printer to clear its buffer by
  //executing the -clear method. 
  [self clear];

  // Last statement of -init.  Tells printer 
  // to return itself, in its current state, to 
  // the caller. 
  return self;
}

//  The -addStringValue method:
//  defines a character string called astring and puts it 
//  into buffer, which is assumed to have been cleared already.
- addStringValue:(const char*)astring
{
  strcat(buffer, astring);
  return self;
}

//  The -print method:
//  prints contents of buffer, then clears buffer.
- print
{
  printf("%s\n",buffer);
  [self clear];
  return self;
}

//  The -clear method: erases contents of buffer
- clear
{
  strcpy(buffer, "");
  return self;
}

@end








