//  ....................................................................
//
//  File: main.m   program: Objc_HelloWorld
//
//  This program sends the message "Hello World!" to the user's 
//  terminal. It is written in Objective C, an object-oriented 
//  dialect of C.  An object-oriented language models reality
//  as a collection of objects which interact by sending, receiving,
//  and responding to messages. It is through this messaging process 
//  that an object-oriented program generates output.
//
//  All Objective C programs have a main.m file that runs things.
//  Additionally, for each class of object there is an 
//  interface file (foo.h) and an implementation file (foo.m).
//
//  This program, Objc_HelloWorld, defines a single object (a printer)
//  which generates the program's output (the "Hello World!" message).
//  It consists of three files: main.m, Printer.h, and Printer.m . 
// 
//  To compile this program, first make the directory containing 
//  the program files your current directory.  Then enter the 
//  following command:
//
//      gcc -o test  -Wno-import main.m Printer.m -lobjc -lpthread
//
//  This will produce an executable file called test. (List the 
//  contents of the directory after entering the command above and
//  you should see test there).
//  
//  To execute the program, enter the following command:
//
//     ./test
//
//  If all goes well there will be no error messages and the words
//  "Hello World!" will appear.
//
//  ....................................................................


//  In Objective C there are common conventions as to
//  whether the first character in a name is upper or lower case. For
//  example in the statements below, Printer (upper case) is a 
//  'class' object whereas printer (lower case) is an 'instance' object, 
//  a specific object in the Printer class.

//  Bring in the file that declares the Printer class.
#import "Printer.h"		

//  Main program begins here.  
int main()
{
  //  Declare printer, an object. Objects in
  //  Objective-C are often declared as type id.
  id printer;
 
  //  Now send some messages to printer.  In Objective C a 
  //  message usually tells an object to do something and is 
  //  always inside square quotes, like this: [dog bark]
  
  //  Our first message creates one instance of Printer, or put more 
  //  strangely, tells printer to create itself.  This step involves 
  //  allocating and initializing memory.
  printer = [[Printer alloc] init];

  //  Several insights can be obtained by studying the statement above. 
  //  For one thing, messages inside messages are OK in Objective-C.
  //  The inside message is acted upon first, so the previous statement is 
  //  equivalent to two separate statements:
  //    printer = [Printer alloc];
  //    printer = [printer init];
  //  The first message takes memory associated with Printer
  //  and allocates it to printer: at this point our generic object, printer,
  //  is specifically associated with the Printer class. The second 
  //  message causes printer to initialize itself by executing
  //  its -init method, defined in file Printer.m .

  //  Next, give printer a specific character string to work with. 
  [printer addStringValue:"Hello World!"];

  //  Tell printer to print the character string.
  [printer print];

  //  We're done with printer, release the memory it used.
  [printer free];

  //  An aside: in complex programs, freeing memory which is still
  //  in use leads to horrendously elusive bugs and failing to free 
  //  memory no longer in use leads to performance-degrading 
  //  'memory leaks'.  

  //  Last statement of main().
  return 1;
}







